part of 'package:zego_uikit_prebuilt_live_streaming/src/live_streaming_controller.dart';

/// @nodoc
mixin ZegoLiveStreamingControllerConnectInvite {
  final _connectInviteController = ZegoLiveStreamingConnectInviteController();

  ZegoLiveStreamingConnectInviteController get connectInvite =>
      _connectInviteController;
}

/// Here are the APIs related to inviting co-hosts to connect.
class ZegoLiveStreamingConnectInviteController {
  /// host invite [audience] to be a co-host
  ///
  /// If [withToast] is set to true, a toast message will be displayed after the request succeeds or fails.
  ///
  /// @return A `Future` that representing whether the request was successful.
  Future<bool> hostInviteAudienceCoHost(
    ZegoUIKitUser audience, {
    bool withToast = false,
  }) async {
    ZegoLoggerService.logInfo(
      'make audience: ${audience.id} to be a co-host',
      tag: 'live streaming',
      subTag: 'controller.connect.invite',
    );

    if (null == _hostManager ||
        null == _connectManager ||
        null == _prebuiltConfig) {
      ZegoLoggerService.logInfo(
        'params is not valid, can not invite, '
        'hostManager:$_hostManager, '
        'connectManager:$_connectManager, '
        'prebuiltConfig:$_prebuiltConfig, ',
        tag: 'live streaming',
        subTag: 'controller.connect.invite',
      );

      return false;
    }

    if (!_isLocalHost) {
      ZegoLoggerService.logInfo(
        'local is not a host, can not invite',
        tag: 'live streaming',
        subTag: 'controller.connect.invite',
      );

      return false;
    }

    final pkBattleState = ZegoLiveStreamingPKBattleManager().state.value;
    if (pkBattleState == ZegoLiveStreamingPKBattleState.inPKBattle ||
        pkBattleState == ZegoLiveStreamingPKBattleState.loading) {
      ZegoLoggerService.logInfo(
        'local in the pk-battle, can not invite',
        tag: 'live streaming',
        subTag: 'controller.connect.invite',
      );

      return false;
    }

    if (_connectManager!.isMaxCoHostReached) {
      _hostManager!.config.onMaxCoHostReached
          ?.call(_prebuiltConfig!.maxCoHostCount);
      ZegoLoggerService.logInfo(
        'co-host max count had reached, can not invite',
        tag: 'live streaming',
        subTag: 'controller.connect.invite',
      );

      return false;
    }

    final targetUserIsInviting =
        _connectManager!.audienceIDsOfInvitingConnect.contains(audience.id);
    final targetUserIsRequesting = -1 !=
        _connectManager!.requestCoHostUsersNotifier.value
            .indexWhere((user) => user.id == audience.id);
    if (targetUserIsInviting || targetUserIsRequesting) {
      ZegoLoggerService.logInfo(
        "you've sent the invitation, please wait for confirmation.",
        tag: 'live streaming',
        subTag: 'controller.connect.invite',
      );

      if (withToast) {
        showError(
          targetUserIsInviting
              ? _prebuiltConfig!.translationText.repeatInviteCoHostFailedToast
              : _prebuiltConfig!.translationText.inviteCoHostFailedToast,
        );
      }

      return false;
    }

    return _connectManager!.inviteAudienceConnect(audience);
  }

  Future<bool> audienceRejectCoHostRequest() async {
    return true;
  }

  Future<bool> audienceAgreeCoHostRequest() async {
    return true;
  }

  ZegoUIKitPrebuiltLiveStreamingConfig? get _prebuiltConfig =>
      ZegoLiveStreamingManagers().hostManager?.config;

  ZegoLiveHostManager? get _hostManager =>
      ZegoLiveStreamingManagers().hostManager;

  ZegoLiveConnectManager? get _connectManager =>
      ZegoLiveStreamingManagers().connectManager;

  bool get _isLocalHost => _hostManager?.isLocalHost ?? false;
}
